{
 "cells": [
  {
   "cell_type": "code",
   "execution_count": 36,
   "metadata": {},
   "outputs": [],
   "source": [
    "import numpy as np\n",
    "from collections import Counter\n",
    "from graphviz import Digraph\n",
    "\n",
    "class Node:\n",
    "\n",
    "    def __init__(self, frequency, letter=None):\n",
    "\n",
    "        self.left=None\n",
    "        self.right=None\n",
    "        self.parent=None\n",
    "        self.frequency = frequency\n",
    "        self.letter = letter if letter is not None else None\n",
    "\n",
    "    def find_top(self):\n",
    "        while(True):\n",
    "            if self.parent is not None:\n",
    "\n",
    "                self = self.parent\n",
    "                \n",
    "            else:\n",
    "                break\n",
    "        return self\n",
    "\n",
    "    def visualize_tree(self):\n",
    "        def add_nodes_edges(self, dot=None):\n",
    "            # Create Digraph object\n",
    "            if dot is None:\n",
    "                dot = Digraph()\n",
    "                dot.node(name=str(self), label=str(self.frequency))\n",
    "\n",
    "            # Add nodes\n",
    "            if self.left:\n",
    "                if self.left.letter == None:\n",
    "                    dot.node(name=str(self.left) ,label=str(self.left.frequency))\n",
    "                else:\n",
    "                    dot.node(name=str(self.left) ,label=str(self.left.letter))\n",
    "                dot.edge(str(self), str(self.left))\n",
    "                dot = add_nodes_edges(self.left, dot=dot)\n",
    "\n",
    "            if self.right:\n",
    "                if self.right.letter == None:\n",
    "                    dot.node(name=str(self.right) ,label=str(self.right.frequency))\n",
    "                else:\n",
    "                    dot.node(name=str(self.right) ,label=str(self.right.letter))\n",
    "\n",
    "                dot.edge(str(self), str(self.right))\n",
    "                dot = add_nodes_edges(self.right, dot=dot)\n",
    "\n",
    "            return dot\n",
    "\n",
    "        dot = add_nodes_edges(self)\n",
    "\n",
    "        # Visualize the graph\n",
    "        display(dot)\n",
    "\n",
    "        return dot\n",
    "    \n",
    "\n",
    "    def display(self):\n",
    "        lines, *_ = self._display_aux()\n",
    "        for line in lines:\n",
    "            print(line)\n",
    "\n",
    "\n",
    "\n",
    "    def _display_aux(self):\n",
    "        \"\"\"Returns list of strings, width, height, and horizontal coordinate of the root.\"\"\"\n",
    "        # No child.\n",
    "        if self.right is None and self.left is None:\n",
    "            line = '%s' % self.frequency\n",
    "            width = len(line)\n",
    "            height = 1\n",
    "            middle = width // 2\n",
    "            return [line], width, height, middle\n",
    "\n",
    "        # Only left child.\n",
    "        if self.right is None:\n",
    "            lines, n, p, x = self.left._display_aux()\n",
    "            s = '%s' % self.frequency\n",
    "            u = len(s)\n",
    "            first_line = (x + 1) * ' ' + (n - x - 1) * '_' + s\n",
    "            second_line = x * ' ' + '/' + (n - x - 1 + u) * ' '\n",
    "            shifted_lines = [line + u * ' ' for line in lines]\n",
    "            return [first_line, second_line] + shifted_lines, n + u, p + 2, n + u // 2\n",
    "\n",
    "        # Only right child.\n",
    "        if self.left is None:\n",
    "            lines, n, p, x = self.right._display_aux()\n",
    "            s = '%s' % self.frequency\n",
    "            u = len(s)\n",
    "            first_line = s + x * '_' + (n - x) * ' '\n",
    "            second_line = (u + x) * ' ' + '\\\\' + (n - x - 1) * ' '\n",
    "            shifted_lines = [u * ' ' + line for line in lines]\n",
    "            return [first_line, second_line] + shifted_lines, n + u, p + 2, u // 2\n",
    "\n",
    "        # Two children.\n",
    "        left, n, p, x = self.left._display_aux()\n",
    "        right, m, q, y = self.right._display_aux()\n",
    "        s = '%s' % self.frequency\n",
    "        u = len(s)\n",
    "        first_line = (x + 1) * ' ' + (n - x - 1) * '_' + s + y * '_' + (m - y) * ' '\n",
    "        second_line = x * ' ' + '/' + (n - x - 1 + u + y) * ' ' + '\\\\' + (m - y - 1) * ' '\n",
    "        if p < q:\n",
    "            left += [n * ' '] * (q - p)\n",
    "        elif q < p:\n",
    "            right += [m * ' '] * (p - q)\n",
    "        zipped_lines = zip(left, right)\n",
    "        lines = [first_line, second_line] + [a + u * ' ' + b for a, b in zipped_lines]\n",
    "        return lines, n + m + u, max(p, q) + 2, n + u // 2\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "'''\n",
    "notes: the root index is zero\n",
    "        for every node with index i the left can be found by 2*i + 1\n",
    "        for the right it can be found by 2*i + 2\n",
    "'''\n",
    "class MinHeap:\n",
    "\n",
    "    def __init__(self,start_size):\n",
    "\n",
    "        self.array = np.empty([start_size], dtype='object')\n",
    "        self.size = 0\n",
    "\n",
    "    def __len__(self):\n",
    "        return len(self.size)\n",
    "\n",
    "    def __str__(self):\n",
    "        mylist =[]\n",
    "        for i in range(self.size):\n",
    "            freq = self.array[i].frequency\n",
    "            if self.array is not None:\n",
    "                letter = self.array[i].letter\n",
    "                mylist.append((freq,letter))\n",
    "            else:\n",
    "                mylist.append((freq,None))\n",
    "        return 'pairlist of frequency&letter: {0}'.format( mylist)\n",
    "\n",
    "\n",
    "    def resize(self):\n",
    "        new_size = 2*len(self.array)\n",
    "        new_array = np.empty([new_size],dtype='object')\n",
    "        for index in range(self.size):\n",
    "            new_array[index] = self.array[index]\n",
    "\n",
    "        self.array = new_array.copy()\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "    def swap(self, index1, index2):\n",
    "        temp_node = self.array[index1]\n",
    "        self.array[index1] = self.array[index2]\n",
    "        self.array[index2] = temp_node\n",
    "\n",
    "\n",
    "\n",
    "    def insert_node(self, node): # inserts at the bottom\n",
    "        if self.size >= len(self.array):\n",
    "            self.resize()\n",
    "\n",
    "        index = self.size\n",
    "        self.array[index] = node\n",
    "        self.size +=1\n",
    "\n",
    "\n",
    "        while (index > 0 ):\n",
    "            if self.array[index].frequency <  self.array[int((index-1)/2)].frequency :\n",
    "                self.swap(index , int((index-1)/2 )) # switchig parent\n",
    "                index = int((index - 1)/2)\n",
    "            else:\n",
    "                break\n",
    "\n",
    "    def pop_node(self): # removes root and returns it also takes leaf and makes it root\n",
    "        root = self.array[0]\n",
    "        index = 0\n",
    "        self.array[0] = self.array[self.size - 1] # move last non-None node to root\n",
    "        self.array[self.size-1] = None # toss out last node\n",
    "        self.size  -= 1\n",
    "\n",
    "        while (index < self.size ):\n",
    "            if (2*index + 1) < self.size and (2*index + 2) < self.size: # check if left and right indices not out of range\n",
    "                if self.array[(2*index + 1)].frequency <  self.array[(2*index + 2)].frequency   \\\n",
    "                    and  self.array[(2*index + 1)].frequency  < self.array[(index)].frequency :\n",
    "\n",
    "                    self.swap((2*index + 1) , index ) # switching left child with parent\n",
    "                    index = (2*index + 1)\n",
    "                elif self.array[(2*index + 1)].frequency  >  self.array[(2*index + 2)].frequency   \\\n",
    "                    and  self.array[(2*index + 2)].frequency  < self.array[(index)].frequency :\n",
    "\n",
    "                    self.swap((2*index + 2) , index ) # switching right child with parent\n",
    "                    index = (2*index + 2)\n",
    "                else:\n",
    "                    break\n",
    "            elif (2*index + 1) < self.size :\n",
    "                if self.array[(2*index + 1)].frequency  < self.array[(index)].frequency :\n",
    "                    self.swap((2*index + 1), index ) # switching left child with parent\n",
    "                    index = (2*index + 1)\n",
    "                else:\n",
    "                    break\n",
    "\n",
    "            elif (2*index + 2) < self.size :\n",
    "                if self.array[(2*index + 2)].frequency  < self.array[(index)].frequency :\n",
    "                    self.swap((2*index + 2), index ) # switching right child with parent\n",
    "                    index = (2*index + 2)\n",
    "                else:\n",
    "                    break\n",
    "            else:\n",
    "                break\n",
    "\n",
    "        return root\n",
    "\n",
    "    #inserts nodes from using values from the dictionary \n",
    "    def heapify(self, new_dictionary):\n",
    "        for letter, frequency in new_dictionary.items():\n",
    "            node = Node(frequency, letter)\n",
    "            self.insert_node(node)\n",
    "        #self.array = self.array[self.array is not np.array(None)]\n",
    "\n",
    "\n",
    "\n",
    "class Huffman:\n",
    "    def __init__(self, letter_list=None):\n",
    "        self.letter_list = letter_list if letter_list is not None else None\n",
    "        self.bin_dict = {}\n",
    "        self.bin_tree_list = []\n",
    "\n",
    "    def encoding(self,root,bits):\n",
    "\n",
    "        if root is None:\n",
    "            return\n",
    "        if root.letter is not None:\n",
    "            self.bin_dict.update({root.letter: bits})\n",
    "        self.encoding(root.left, bits + '0')\n",
    "        self.encoding(root.right, bits + '1')\n",
    "\n",
    "    def bin_tree(self,root):\n",
    "        if root is  None:\n",
    "            return\n",
    "        if root.letter is not None:\n",
    "            self.bin_tree_list.append('1')\n",
    "            self.bin_tree_list.append(root.letter)\n",
    "        elif root.letter is None:\n",
    "            self.bin_tree_list.append('0')\n",
    "            self.bin_tree(root.left)\n",
    "            self.bin_tree(root.right)\n",
    "\n",
    "\n",
    "    def huffing(self):\n",
    "        frequency_dict = Counter(self.letter_list)\n",
    "        print(frequency_dict)\n",
    "        heap = MinHeap(len(frequency_dict))\n",
    "        heap.heapify(frequency_dict)\n",
    "        merge_heap = MinHeap(len(frequency_dict))\n",
    "\n",
    "        for count in range(heap.size-1):\n",
    "\n",
    "            \n",
    "            if heap.size > 1: \n",
    "                lowest = heap.pop_node()\n",
    "                next_lowest = heap.pop_node()\n",
    "                node  = Node(lowest.frequency + next_lowest.frequency)\n",
    "                node.left = lowest\n",
    "                node.right = next_lowest\n",
    "                merge_heap.insert_node(node)\n",
    "            else:\n",
    "                if heap.size > 0:\n",
    "                    lowest = heap.pop_node()\n",
    "                    merge_heap.insert_node(lowest)\n",
    "                \n",
    "                \n",
    "                for next_count in range(merge_heap.size-1):\n",
    "                    lowest = merge_heap.pop_node()\n",
    "                    next_lowest = merge_heap.pop_node()\n",
    "                    print(lowest.frequency)\n",
    "                    node  = Node(lowest.frequency + next_lowest.frequency)\n",
    "                    node.left = lowest\n",
    "                    node.right = next_lowest\n",
    "                    merge_heap.insert_node(node)\n",
    "                \n",
    "                    \n",
    "                    \n",
    "                    \n",
    "                \n",
    "                \n",
    "        \n",
    "        dot = merge_heap.array[0].visualize_tree()\n",
    "\n",
    "        self.bin_tree(merge_heap.array[0])\n",
    "        self.encoding(merge_heap.array[0],'')\n",
    "\n",
    "\n",
    "\n",
    "    def write_code(self):\n",
    "\n",
    "        with open('huffman.txt', 'w') as writer:\n",
    "            for value in self.bin_tree_list:\n",
    "                writer.write(value)\n",
    "\n",
    "            writer.write('|')\n",
    "            for letter in self.letter_list:\n",
    "                code = self.bin_dict[letter]\n",
    "                writer.write(code)\n",
    "            self.bin_tree_list = []\n",
    "            self.bin_dict = {}\n",
    "\n",
    "\n",
    "    def decode(self):\n",
    "\n",
    "        root = Node(42)\n",
    "        iter_code = iter(self.bin_tree_list)\n",
    "        value = next(iter_code)\n",
    "        try:\n",
    "            for i in range(len(self.bin_tree_list)):\n",
    "                value = next(iter_code)\n",
    "\n",
    "                if root.left is not None and root.right is not None:\n",
    "                    root = root.parent\n",
    "\n",
    "                if value == '0':\n",
    "                    node = Node(1)\n",
    "                    if root.left is None:\n",
    "                        root.left = node\n",
    "                    else:\n",
    "                        root.right = node\n",
    "\n",
    "                    node.parent = root\n",
    "                    root = node\n",
    "\n",
    "\n",
    "\n",
    "                elif value == '1':\n",
    "                    value = next(iter_code)\n",
    "\n",
    "                    node = Node(2,value)\n",
    "\n",
    "                    if root.left is None:\n",
    "                        root.left = node\n",
    "                    else:\n",
    "                        root.right = node\n",
    "                    node.parent = root\n",
    "\n",
    "\n",
    "\n",
    "        except StopIteration:\n",
    "                    pass\n",
    "        root = root.find_top()\n",
    "\n",
    "\n",
    "        return root\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "    def read_code(self):\n",
    "        tree = []\n",
    "        bin = []\n",
    "        letters = []\n",
    "\n",
    "\n",
    "        with open('huffman.txt', 'r') as reader:\n",
    "            contents = list(reader.read())\n",
    "            copy_contents = contents.copy()\n",
    "        for tree_value in contents:\n",
    "            if (tree_value) == '|':\n",
    "                copy_contents.pop(0)\n",
    "                break\n",
    "            else:\n",
    "                self.bin_tree_list.append(tree_value)\n",
    "                copy_contents.pop(0)\n",
    "\n",
    "\n",
    "        new_tree = self.decode()\n",
    "        #new_tree.display()\n",
    "        for bit in copy_contents:\n",
    "\n",
    "\n",
    "\n",
    "            if  bit == '0' :\n",
    "                new_tree = new_tree.left\n",
    "                if new_tree.left is None and new_tree.right == None:\n",
    "                    letters.append(new_tree.letter)\n",
    "                    new_tree = new_tree.find_top()\n",
    "\n",
    "\n",
    "            elif bit == '1':\n",
    "                new_tree = new_tree.right\n",
    "                if new_tree.left is None and new_tree.right == None:\n",
    "                    letters.append(new_tree.letter)\n",
    "                    new_tree = new_tree.find_top()\n",
    "\n",
    "\n",
    "        print(letters)\n",
    "        return letters\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 43,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Counter({'e': 4, ' ': 3, 'o': 3, 'f': 2, 'd': 2, 'm': 2, 'r': 1, '\\n': 1})\n",
      "2\n",
      "5\n",
      "7\n"
     ]
    },
    {
     "data": {
      "image/svg+xml": [
       "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n",
       "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\"\n",
       " \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n",
       "<!-- Generated by graphviz version 2.40.1 (20161225.0304)\n",
       " -->\n",
       "<!-- Title: %3 Pages: 1 -->\n",
       "<svg width=\"422pt\" height=\"332pt\"\n",
       " viewBox=\"0.00 0.00 422.00 332.00\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\">\n",
       "<g id=\"graph0\" class=\"graph\" transform=\"scale(1 1) rotate(0) translate(4 328)\">\n",
       "<title>%3</title>\n",
       "<polygon fill=\"#ffffff\" stroke=\"transparent\" points=\"-4,4 -4,-328 418,-328 418,4 -4,4\"/>\n",
       "<!-- __main__.Node object at 0x7f897c3f8208 -->\n",
       "<g id=\"node1\" class=\"node\">\n",
       "<title>__main__.Node object at 0x7f897c3f8208</title>\n",
       "<ellipse fill=\"none\" stroke=\"#000000\" cx=\"135\" cy=\"-306\" rx=\"27\" ry=\"18\"/>\n",
       "<text text-anchor=\"middle\" x=\"135\" y=\"-302.3\" font-family=\"Times,serif\" font-size=\"14.00\" fill=\"#000000\">18</text>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f8940 -->\n",
       "<g id=\"node2\" class=\"node\">\n",
       "<title>__main__.Node object at 0x7f897c3f8940</title>\n",
       "<ellipse fill=\"none\" stroke=\"#000000\" cx=\"99\" cy=\"-234\" rx=\"27\" ry=\"18\"/>\n",
       "<text text-anchor=\"middle\" x=\"99\" y=\"-230.3\" font-family=\"Times,serif\" font-size=\"14.00\" fill=\"#000000\">7</text>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f8208&#45;&gt;__main__.Node object at 0x7f897c3f8940 -->\n",
       "<g id=\"edge1\" class=\"edge\">\n",
       "<title>__main__.Node object at 0x7f897c3f8208&#45;&gt;__main__.Node object at 0x7f897c3f8940</title>\n",
       "<path fill=\"none\" stroke=\"#000000\" d=\"M126.2854,-288.5708C122.0403,-280.0807 116.8464,-269.6929 112.1337,-260.2674\"/>\n",
       "<polygon fill=\"#000000\" stroke=\"#000000\" points=\"115.237,-258.6477 107.6343,-251.2687 108.976,-261.7782 115.237,-258.6477\"/>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f87f0 -->\n",
       "<g id=\"node5\" class=\"node\">\n",
       "<title>__main__.Node object at 0x7f897c3f87f0</title>\n",
       "<ellipse fill=\"none\" stroke=\"#000000\" cx=\"171\" cy=\"-234\" rx=\"27\" ry=\"18\"/>\n",
       "<text text-anchor=\"middle\" x=\"171\" y=\"-230.3\" font-family=\"Times,serif\" font-size=\"14.00\" fill=\"#000000\">11</text>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f8208&#45;&gt;__main__.Node object at 0x7f897c3f87f0 -->\n",
       "<g id=\"edge4\" class=\"edge\">\n",
       "<title>__main__.Node object at 0x7f897c3f8208&#45;&gt;__main__.Node object at 0x7f897c3f87f0</title>\n",
       "<path fill=\"none\" stroke=\"#000000\" d=\"M143.7146,-288.5708C147.9597,-280.0807 153.1536,-269.6929 157.8663,-260.2674\"/>\n",
       "<polygon fill=\"#000000\" stroke=\"#000000\" points=\"161.024,-261.7782 162.3657,-251.2687 154.763,-258.6477 161.024,-261.7782\"/>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f8240 -->\n",
       "<g id=\"node3\" class=\"node\">\n",
       "<title>__main__.Node object at 0x7f897c3f8240</title>\n",
       "<ellipse fill=\"none\" stroke=\"#000000\" cx=\"27\" cy=\"-162\" rx=\"27\" ry=\"18\"/>\n",
       "<text text-anchor=\"middle\" x=\"27\" y=\"-158.3\" font-family=\"Times,serif\" font-size=\"14.00\" fill=\"#000000\"> </text>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f8940&#45;&gt;__main__.Node object at 0x7f897c3f8240 -->\n",
       "<g id=\"edge2\" class=\"edge\">\n",
       "<title>__main__.Node object at 0x7f897c3f8940&#45;&gt;__main__.Node object at 0x7f897c3f8240</title>\n",
       "<path fill=\"none\" stroke=\"#000000\" d=\"M83.7307,-218.7307C73.803,-208.803 60.6847,-195.6847 49.5637,-184.5637\"/>\n",
       "<polygon fill=\"#000000\" stroke=\"#000000\" points=\"51.7933,-181.8436 42.2473,-177.2473 46.8436,-186.7933 51.7933,-181.8436\"/>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f8470 -->\n",
       "<g id=\"node4\" class=\"node\">\n",
       "<title>__main__.Node object at 0x7f897c3f8470</title>\n",
       "<ellipse fill=\"none\" stroke=\"#000000\" cx=\"99\" cy=\"-162\" rx=\"27\" ry=\"18\"/>\n",
       "<text text-anchor=\"middle\" x=\"99\" y=\"-158.3\" font-family=\"Times,serif\" font-size=\"14.00\" fill=\"#000000\">e</text>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f8940&#45;&gt;__main__.Node object at 0x7f897c3f8470 -->\n",
       "<g id=\"edge3\" class=\"edge\">\n",
       "<title>__main__.Node object at 0x7f897c3f8940&#45;&gt;__main__.Node object at 0x7f897c3f8470</title>\n",
       "<path fill=\"none\" stroke=\"#000000\" d=\"M99,-215.8314C99,-208.131 99,-198.9743 99,-190.4166\"/>\n",
       "<polygon fill=\"#000000\" stroke=\"#000000\" points=\"102.5001,-190.4132 99,-180.4133 95.5001,-190.4133 102.5001,-190.4132\"/>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f8550 -->\n",
       "<g id=\"node6\" class=\"node\">\n",
       "<title>__main__.Node object at 0x7f897c3f8550</title>\n",
       "<ellipse fill=\"none\" stroke=\"#000000\" cx=\"171\" cy=\"-162\" rx=\"27\" ry=\"18\"/>\n",
       "<text text-anchor=\"middle\" x=\"171\" y=\"-158.3\" font-family=\"Times,serif\" font-size=\"14.00\" fill=\"#000000\">5</text>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f87f0&#45;&gt;__main__.Node object at 0x7f897c3f8550 -->\n",
       "<g id=\"edge5\" class=\"edge\">\n",
       "<title>__main__.Node object at 0x7f897c3f87f0&#45;&gt;__main__.Node object at 0x7f897c3f8550</title>\n",
       "<path fill=\"none\" stroke=\"#000000\" d=\"M171,-215.8314C171,-208.131 171,-198.9743 171,-190.4166\"/>\n",
       "<polygon fill=\"#000000\" stroke=\"#000000\" points=\"174.5001,-190.4132 171,-180.4133 167.5001,-190.4133 174.5001,-190.4132\"/>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f8160 -->\n",
       "<g id=\"node9\" class=\"node\">\n",
       "<title>__main__.Node object at 0x7f897c3f8160</title>\n",
       "<ellipse fill=\"none\" stroke=\"#000000\" cx=\"243\" cy=\"-162\" rx=\"27\" ry=\"18\"/>\n",
       "<text text-anchor=\"middle\" x=\"243\" y=\"-158.3\" font-family=\"Times,serif\" font-size=\"14.00\" fill=\"#000000\">6</text>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f87f0&#45;&gt;__main__.Node object at 0x7f897c3f8160 -->\n",
       "<g id=\"edge8\" class=\"edge\">\n",
       "<title>__main__.Node object at 0x7f897c3f87f0&#45;&gt;__main__.Node object at 0x7f897c3f8160</title>\n",
       "<path fill=\"none\" stroke=\"#000000\" d=\"M186.2693,-218.7307C196.197,-208.803 209.3153,-195.6847 220.4363,-184.5637\"/>\n",
       "<polygon fill=\"#000000\" stroke=\"#000000\" points=\"223.1564,-186.7933 227.7527,-177.2473 218.2067,-181.8436 223.1564,-186.7933\"/>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f8c88 -->\n",
       "<g id=\"node7\" class=\"node\">\n",
       "<title>__main__.Node object at 0x7f897c3f8c88</title>\n",
       "<ellipse fill=\"none\" stroke=\"#000000\" cx=\"99\" cy=\"-90\" rx=\"27\" ry=\"18\"/>\n",
       "<text text-anchor=\"middle\" x=\"99\" y=\"-86.3\" font-family=\"Times,serif\" font-size=\"14.00\" fill=\"#000000\">d</text>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f8550&#45;&gt;__main__.Node object at 0x7f897c3f8c88 -->\n",
       "<g id=\"edge6\" class=\"edge\">\n",
       "<title>__main__.Node object at 0x7f897c3f8550&#45;&gt;__main__.Node object at 0x7f897c3f8c88</title>\n",
       "<path fill=\"none\" stroke=\"#000000\" d=\"M155.7307,-146.7307C145.803,-136.803 132.6847,-123.6847 121.5637,-112.5637\"/>\n",
       "<polygon fill=\"#000000\" stroke=\"#000000\" points=\"123.7933,-109.8436 114.2473,-105.2473 118.8436,-114.7933 123.7933,-109.8436\"/>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f82b0 -->\n",
       "<g id=\"node8\" class=\"node\">\n",
       "<title>__main__.Node object at 0x7f897c3f82b0</title>\n",
       "<ellipse fill=\"none\" stroke=\"#000000\" cx=\"171\" cy=\"-90\" rx=\"27\" ry=\"18\"/>\n",
       "<text text-anchor=\"middle\" x=\"171\" y=\"-86.3\" font-family=\"Times,serif\" font-size=\"14.00\" fill=\"#000000\">o</text>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f8550&#45;&gt;__main__.Node object at 0x7f897c3f82b0 -->\n",
       "<g id=\"edge7\" class=\"edge\">\n",
       "<title>__main__.Node object at 0x7f897c3f8550&#45;&gt;__main__.Node object at 0x7f897c3f82b0</title>\n",
       "<path fill=\"none\" stroke=\"#000000\" d=\"M171,-143.8314C171,-136.131 171,-126.9743 171,-118.4166\"/>\n",
       "<polygon fill=\"#000000\" stroke=\"#000000\" points=\"174.5001,-118.4132 171,-108.4133 167.5001,-118.4133 174.5001,-118.4132\"/>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f89b0 -->\n",
       "<g id=\"node10\" class=\"node\">\n",
       "<title>__main__.Node object at 0x7f897c3f89b0</title>\n",
       "<ellipse fill=\"none\" stroke=\"#000000\" cx=\"243\" cy=\"-90\" rx=\"27\" ry=\"18\"/>\n",
       "<text text-anchor=\"middle\" x=\"243\" y=\"-86.3\" font-family=\"Times,serif\" font-size=\"14.00\" fill=\"#000000\">2</text>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f8160&#45;&gt;__main__.Node object at 0x7f897c3f89b0 -->\n",
       "<g id=\"edge9\" class=\"edge\">\n",
       "<title>__main__.Node object at 0x7f897c3f8160&#45;&gt;__main__.Node object at 0x7f897c3f89b0</title>\n",
       "<path fill=\"none\" stroke=\"#000000\" d=\"M243,-143.8314C243,-136.131 243,-126.9743 243,-118.4166\"/>\n",
       "<polygon fill=\"#000000\" stroke=\"#000000\" points=\"246.5001,-118.4132 243,-108.4133 239.5001,-118.4133 246.5001,-118.4132\"/>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f8cf8 -->\n",
       "<g id=\"node13\" class=\"node\">\n",
       "<title>__main__.Node object at 0x7f897c3f8cf8</title>\n",
       "<ellipse fill=\"none\" stroke=\"#000000\" cx=\"315\" cy=\"-90\" rx=\"27\" ry=\"18\"/>\n",
       "<text text-anchor=\"middle\" x=\"315\" y=\"-86.3\" font-family=\"Times,serif\" font-size=\"14.00\" fill=\"#000000\">4</text>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f8160&#45;&gt;__main__.Node object at 0x7f897c3f8cf8 -->\n",
       "<g id=\"edge12\" class=\"edge\">\n",
       "<title>__main__.Node object at 0x7f897c3f8160&#45;&gt;__main__.Node object at 0x7f897c3f8cf8</title>\n",
       "<path fill=\"none\" stroke=\"#000000\" d=\"M258.2693,-146.7307C268.197,-136.803 281.3153,-123.6847 292.4363,-112.5637\"/>\n",
       "<polygon fill=\"#000000\" stroke=\"#000000\" points=\"295.1564,-114.7933 299.7527,-105.2473 290.2067,-109.8436 295.1564,-114.7933\"/>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f8390 -->\n",
       "<g id=\"node11\" class=\"node\">\n",
       "<title>__main__.Node object at 0x7f897c3f8390</title>\n",
       "<ellipse fill=\"none\" stroke=\"#000000\" cx=\"171\" cy=\"-18\" rx=\"27\" ry=\"18\"/>\n",
       "<text text-anchor=\"middle\" x=\"171\" y=\"-14.3\" font-family=\"Times,serif\" font-size=\"14.00\" fill=\"#000000\">r</text>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f89b0&#45;&gt;__main__.Node object at 0x7f897c3f8390 -->\n",
       "<g id=\"edge10\" class=\"edge\">\n",
       "<title>__main__.Node object at 0x7f897c3f89b0&#45;&gt;__main__.Node object at 0x7f897c3f8390</title>\n",
       "<path fill=\"none\" stroke=\"#000000\" d=\"M227.7307,-74.7307C217.803,-64.803 204.6847,-51.6847 193.5637,-40.5637\"/>\n",
       "<polygon fill=\"#000000\" stroke=\"#000000\" points=\"195.7933,-37.8436 186.2473,-33.2473 190.8436,-42.7933 195.7933,-37.8436\"/>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f8978 -->\n",
       "<g id=\"node12\" class=\"node\">\n",
       "<title>__main__.Node object at 0x7f897c3f8978</title>\n",
       "<ellipse fill=\"none\" stroke=\"#000000\" cx=\"243\" cy=\"-18\" rx=\"27\" ry=\"18\"/>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f89b0&#45;&gt;__main__.Node object at 0x7f897c3f8978 -->\n",
       "<g id=\"edge11\" class=\"edge\">\n",
       "<title>__main__.Node object at 0x7f897c3f89b0&#45;&gt;__main__.Node object at 0x7f897c3f8978</title>\n",
       "<path fill=\"none\" stroke=\"#000000\" d=\"M243,-71.8314C243,-64.131 243,-54.9743 243,-46.4166\"/>\n",
       "<polygon fill=\"#000000\" stroke=\"#000000\" points=\"246.5001,-46.4132 243,-36.4133 239.5001,-46.4133 246.5001,-46.4132\"/>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f8748 -->\n",
       "<g id=\"node14\" class=\"node\">\n",
       "<title>__main__.Node object at 0x7f897c3f8748</title>\n",
       "<ellipse fill=\"none\" stroke=\"#000000\" cx=\"315\" cy=\"-18\" rx=\"27\" ry=\"18\"/>\n",
       "<text text-anchor=\"middle\" x=\"315\" y=\"-14.3\" font-family=\"Times,serif\" font-size=\"14.00\" fill=\"#000000\">f</text>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f8cf8&#45;&gt;__main__.Node object at 0x7f897c3f8748 -->\n",
       "<g id=\"edge13\" class=\"edge\">\n",
       "<title>__main__.Node object at 0x7f897c3f8cf8&#45;&gt;__main__.Node object at 0x7f897c3f8748</title>\n",
       "<path fill=\"none\" stroke=\"#000000\" d=\"M315,-71.8314C315,-64.131 315,-54.9743 315,-46.4166\"/>\n",
       "<polygon fill=\"#000000\" stroke=\"#000000\" points=\"318.5001,-46.4132 315,-36.4133 311.5001,-46.4133 318.5001,-46.4132\"/>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f84a8 -->\n",
       "<g id=\"node15\" class=\"node\">\n",
       "<title>__main__.Node object at 0x7f897c3f84a8</title>\n",
       "<ellipse fill=\"none\" stroke=\"#000000\" cx=\"387\" cy=\"-18\" rx=\"27\" ry=\"18\"/>\n",
       "<text text-anchor=\"middle\" x=\"387\" y=\"-14.3\" font-family=\"Times,serif\" font-size=\"14.00\" fill=\"#000000\">m</text>\n",
       "</g>\n",
       "<!-- __main__.Node object at 0x7f897c3f8cf8&#45;&gt;__main__.Node object at 0x7f897c3f84a8 -->\n",
       "<g id=\"edge14\" class=\"edge\">\n",
       "<title>__main__.Node object at 0x7f897c3f8cf8&#45;&gt;__main__.Node object at 0x7f897c3f84a8</title>\n",
       "<path fill=\"none\" stroke=\"#000000\" d=\"M330.2693,-74.7307C340.197,-64.803 353.3153,-51.6847 364.4363,-40.5637\"/>\n",
       "<polygon fill=\"#000000\" stroke=\"#000000\" points=\"367.1564,-42.7933 371.7527,-33.2473 362.2067,-37.8436 367.1564,-42.7933\"/>\n",
       "</g>\n",
       "</g>\n",
       "</svg>\n"
      ],
      "text/plain": [
       "<graphviz.dot.Digraph at 0x7f897c3f8828>"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    },
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "{' ': '00', 'e': '01', 'd': '100', 'o': '101', 'r': '1100', '\\n': '1101', 'f': '1110', 'm': '1111'}\n"
     ]
    }
   ],
   "source": [
    "#main\n",
    "\n",
    "with open('message.txt', 'r') as reader:\n",
    "    contents = list(reader.read())\n",
    "\n",
    "\n",
    "#print(contents)\n",
    "huffman = Huffman(contents)\n",
    "huffman.huffing()\n",
    "print(huffman.bin_dict)\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 44,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "['f', 'e', 'e', 'd', ' ', 'm', 'e', ' ', 'm', 'o', 'r', 'e', ' ', 'f', 'o', 'o', 'd', '\\n']\n"
     ]
    }
   ],
   "source": [
    "huffman.write_code()\n",
    "letters = huffman.read_code()\n",
    "\n",
    "with open('decoded.txt', 'w') as writer:\n",
    "    for value in letters:\n",
    "        writer.write(value)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": []
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.7.3"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 2
}
